package Locale::Codes::Script_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Thu Mar  1 08:11:47 EST 2012

=pod

=head1 NAME

Locale::Codes::Script_Retired - retired script codes for the Locale::Codes::Script module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::Script module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
require 5.002;

our($VERSION);
$VERSION='3.21';

$Locale::Codes::Retired{'script'}{'num'}{'code'} = {
};

$Locale::Codes::Retired{'script'}{'alpha'}{'code'} = {
};

$Locale::Codes::Retired{'script'}{'num'}{'name'} = {
   q(tiruta) => [ q(326), q(Tiruta) ],
};

$Locale::Codes::Retired{'script'}{'alpha'}{'name'} = {
   q(sindhi) => [ q(Sind), q(Sindhi) ],
   q(miao (pollard)) => [ q(Plrd), q(Miao (Pollard)) ],
   q(ol chiki (ol cemet, ol, santali)) => [ q(Olck), q(Ol Chiki (Ol Cemet, Ol, Santali)) ],
   q(hangul (hangul, hangeul)) => [ q(Hang), q(Hangul (Hangul, Hangeul)) ],
   q(sharada, sarada) => [ q(Shrd), q(Sharada, Sarada) ],
   q(khudawadi, sindhi) => [ q(Sind), q(Khudawadi, Sindhi) ],
   q(myanmar (burmese)) => [ q(Mymr), q(Myanmar (Burmese)) ],
   q(tifinagh (berber)) => [ q(Tfng), q(Tifinagh (Berber)) ],
   q(moon (moon code, moon script, moon type)) => [ q(Moon), q(Moon (Moon code, Moon script, Moon type)) ],
   q(cuneiform, sumero-akkadian) => [ q(Xsux), q(Cuneiform, Sumero-Akkadian) ],
   q(warang citi (varang kshiti)) => [ q(Wara), q(Warang Citi (Varang Kshiti)) ],
   q(ethiopic (geez)) => [ q(Ethi), q(Ethiopic (Geez)) ],
   q(tagalog (baybayin, alibata)) => [ q(Tglg), q(Tagalog (Baybayin, Alibata)) ],
   q(deseret (mormon)) => [ q(Dsrt), q(Deseret (Mormon)) ],
   q((alias for hiragana + katakana)) => [ q(Hrkt), q((alias for Hiragana + Katakana)) ],
   q(mandaic, mandaean) => [ q(Mand), q(Mandaic, Mandaean) ],
   q(devanagari (nagari)) => [ q(Deva), q(Devanagari (Nagari)) ],
   q(tai tham (lanna)) => [ q(Lana), q(Tai Tham (Lanna)) ],
   q(shavian (shaw)) => [ q(Shaw), q(Shavian (Shaw)) ],
   q(indus (harappan)) => [ q(Inds), q(Indus (Harappan)) ],
   q(lisu (fraser)) => [ q(Lisu), q(Lisu (Fraser)) ],
   q(lepcha (rong)) => [ q(Lepc), q(Lepcha (Rong)) ],
   q(mro, mru) => [ q(Mroo), q(Mro, Mru) ],
   q(meitei mayek (meithei, meetei)) => [ q(Mtei), q(Meitei Mayek (Meithei, Meetei)) ],
   q(tiruta) => [ q(Tirh), q(Tiruta) ],
   q(nakhi geba ('na-'khi ggo-baw, naxi geba)) => [ q(Nkgb), q(Nakhi Geba ('Na-'Khi Ggo-baw, Naxi Geba)) ],
   q(old north arabian (ancient north arabian)) => [ q(Narb), q(Old North Arabian (Ancient North Arabian)) ],
   q(takri, takri, tankri) => [ q(Takr), q(Takri, Takri, Tankri) ],
   q(rejang (redjang, kaganga)) => [ q(Rjng), q(Rejang (Redjang, Kaganga)) ],
   q(han (hanzi, kanji, hanja)) => [ q(Hani), q(Han (Hanzi, Kanji, Hanja)) ],
   q(old turkic, orkhon runic) => [ q(Orkh), q(Old Turkic, Orkhon Runic) ],
   q(hanunoo (hanunoo)) => [ q(Hano), q(Hanunoo (Hanunoo)) ],
   q(duployan shorthand, duployan stenography) => [ q(Dupl), q(Duployan shorthand, Duployan stenography) ],
};


1;
