+{
   locale_version => 0.88,
   entry => <<'ENTRY', # for DUCET v6.1.0
00E4      ; [.15D5.0020.0002.00E4] # LATIN SMALL LETTER A WITH DIAERESIS
0061 0308 ; [.15D5.0020.0002.00E4] # LATIN SMALL LETTER A WITH DIAERESIS
00C4      ; [.15D5.0020.0008.00C4] # LATIN CAPITAL LETTER A WITH DIAERESIS
0041 0308 ; [.15D5.0020.0008.00C4] # LATIN CAPITAL LETTER A WITH DIAERESIS
01DF      ; [.15D5.0020.0002.00E4][.0000.005B.0002.0304] # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
01DE      ; [.15D5.0020.0008.00C4][.0000.005B.0002.0304] # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
010D      ; [.1603.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1603.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1603.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1603.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0063 0068 ; [.169A.0020.0002.0063] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0063 0048 ; [.169A.0020.0007.0063][.0000.0000.0002.0000] # <LATIN SMALL LETTER C, LATIN CAPITAL LETTER H>
0043 0068 ; [.169A.0020.0007.0043][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.169A.0020.0008.0043] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
00F4      ; [.1757.0020.0002.00F4] # LATIN SMALL LETTER O WITH CIRCUMFLEX
006F 0302 ; [.1757.0020.0002.00F4] # LATIN SMALL LETTER O WITH CIRCUMFLEX
00D4      ; [.1757.0020.0008.00D4] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
004F 0302 ; [.1757.0020.0008.00D4] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
1ED3      ; [.1757.0020.0002.00F4][.0000.0035.0002.0300] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE
1ED2      ; [.1757.0020.0008.00D4][.0000.0035.0002.0300] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE
1ED1      ; [.1757.0020.0002.00F4][.0000.0032.0002.0301] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE
1ED0      ; [.1757.0020.0008.00D4][.0000.0032.0002.0301] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE
1ED7      ; [.1757.0020.0002.00F4][.0000.004E.0002.0303] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE
1ED6      ; [.1757.0020.0008.00D4][.0000.004E.0002.0303] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE
1ED5      ; [.1757.0020.0002.00F4][.0000.0064.0002.0309] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
1ED4      ; [.1757.0020.0008.00D4][.0000.0064.0002.0309] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
1ED9      ; [.1757.0020.0002.00F4][.0000.0070.0002.0323] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1ED8      ; [.1757.0020.0008.00D4][.0000.0070.0002.0323] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
0159      ; [.17A3.0020.0002.0159] # LATIN SMALL LETTER R WITH CARON
0072 030C ; [.17A3.0020.0002.0159] # LATIN SMALL LETTER R WITH CARON
0158      ; [.17A3.0020.0008.0158] # LATIN CAPITAL LETTER R WITH CARON
0052 030C ; [.17A3.0020.0008.0158] # LATIN CAPITAL LETTER R WITH CARON
0161      ; [.17D9.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.17D9.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0160      ; [.17D9.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.17D9.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.187B.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.187B.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.187B.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.187B.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
