package Locale::Codes::LangFam_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Tue Sep 27 15:40:31 EDT 2011

=pod

=head1 NAME

Locale::Codes::LangFam_Retired - retired langfam codes for the Locale::Codes::LangFam module

=head1 SYNOPSIS

This module contains data used by the Locale::Codes::LangFam module. It is
not intended to be used directly, and contains no calleable routines.

=head1 AUTHOR

See Locale::Codes for full author history.

Currently maintained by Sullivan Beck (sbeck@cpan.org).

=head1 COPYRIGHT

   Copyright (c) 1997-2001 Canon Research Centre Europe (CRE).
   Copyright (c) 2001-2010 Neil Bowers
   Copyright (c) 2010-2012 Sullivan Beck

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use strict;
use warnings;
require 5.002;

our($VERSION);
$VERSION='3.21';

$Locale::Codes::Retired{'langfam'}{'alpha'}{'code'} = {
};

$Locale::Codes::Retired{'langfam'}{'alpha'}{'name'} = {
};


1;
